{% load static %}

<select name="{{ widget.name }}"{% include "django/forms/widgets/attrs.html" %}>{% for group_name, group_choices, group_index in widget.optgroups %}{% if group_name %}
  <optgroup label="{{ group_name }}">{% endif %}{% for option in group_choices %}
  {% include option.template_name with widget=option %}{% endfor %}{% if group_name %}
  </optgroup>{% endif %}{% endfor %}
</select>

<div class="generic-foreign-key-fk-field"></div>

<a href="#" title="Clear" class="clear-generic-field">
  <img src="{% static 'jnt_django_toolbox/images/icon-broom.svg' %}"/>
</a>

<a href="" title="Edit" id="edit-generic-button-{{ widget.name }}" class="related-widget-wrapper-link generic-foreign-field-wrapper-href edit-generic-field">
  <img src="{% static 'admin/img/icon-changelink.svg' %}"/>
</a>

