from .autocomplete_fields import AutocompleteFieldsAdminMixin
from .generic_foreign_key import GenericForeignKeyAdminMixin
from .generic_foreign_key_inline import GenericForeignKeyInlineAdminMixin
from .readonly_widgets import ReadonlyWidgetsMixin
from .clickable_links import ClickableLinksAdminMixin
from .autocomplete_changelist_filters import (
    AutocompleteChangelistFiltersAdminMixin,
)
from .autocomplete import AutocompleteAdminMixin
